@@ -36,45 +36,47 @@
 #define MAXMOUNTPOINTS			(1<<8)
 #define MAXPERMITTEDHOOKFILES		(1<<8)
 
-#ifdef __CLSYNC_COMMON_H
-#	ifndef DEFAULT_NOTIFYENGINE
-#		ifdef __linux__
+#ifndef PIC
+#	ifdef __CLSYNC_COMMON_H
+#		ifndef DEFAULT_NOTIFYENGINE
+#			ifdef __linux__
+#				ifdef INOTIFY_SUPPORT
+#					define DEFAULT_NOTIFYENGINE	NE_INOTIFY
+#				endif
+#			endif
+#		endif
+#		ifndef DEFAULT_NOTIFYENGINE
+#			if __FreeBSD__ | __FreeBSD_kernel__
+#				ifdef KQUEUE_SUPPORT
+#					define DEFAULT_NOTIFYENGINE	NE_KQUEUE
+#				endif
+#			endif
+#		endif
+#		ifndef DEFAULT_NOTIFYENGINE
 #			ifdef INOTIFY_SUPPORT
 #				define DEFAULT_NOTIFYENGINE	NE_INOTIFY
 #			endif
 #		endif
-#	endif
-#	ifndef DEFAULT_NOTIFYENGINE
-#		ifdef __FreeBSD__
+#		ifndef DEFAULT_NOTIFYENGINE
+#			ifdef GIO_SUPPORT
+#				define DEFAULT_NOTIFYENGINE	NE_GIO
+#			endif
+#		endif
+#		ifndef DEFAULT_NOTIFYENGINE
 #			ifdef KQUEUE_SUPPORT
 #				define DEFAULT_NOTIFYENGINE	NE_KQUEUE
 #			endif
 #		endif
-#	endif
-#	ifndef DEFAULT_NOTIFYENGINE
-#		ifdef INOTIFY_SUPPORT
-#			define DEFAULT_NOTIFYENGINE	NE_INOTIFY
+#		ifndef DEFAULT_NOTIFYENGINE
+#			ifdef BSM_SUPPORT
+#				define DEFAULT_NOTIFYENGINE	NE_BSM
+#			endif
 #		endif
-#	endif
-#	ifndef DEFAULT_NOTIFYENGINE
-#		ifdef GIO_SUPPORT
-#			define DEFAULT_NOTIFYENGINE	NE_GIO
+#		ifndef DEFAULT_NOTIFYENGINE
+#			error No monitor subsystem supported
+#			define  DEFAULT_NOTIFYENGINE		NE_UNDEFINED
 #		endif
 #	endif
-#	ifndef DEFAULT_NOTIFYENGINE
-#		ifdef KQUEUE_SUPPORT
-#			define DEFAULT_NOTIFYENGINE	NE_KQUEUE
-#		endif
-#	endif
-#	ifndef DEFAULT_NOTIFYENGINE
-#		ifdef BSM_SUPPORT
-#			define DEFAULT_NOTIFYENGINE	NE_BSM
-#		endif
-#	endif
-#	ifndef DEFAULT_NOTIFYENGINE
-#		warning No default monitor subsystem is set
-#		define  DEFAULT_NOTIFYENGINE		NE_UNDEFINED
-#	endif
 #endif
 
 #define DEFAULT_RULES_PERM		RA_ALL
@@ -213,6 +215,8 @@ expire-after:20M\n\
 // Upper limit
 #define HL_LOCK_AUTO_LIMIT_HIGH		(1<<20)
 
+#define HL_LOCK_NONPRIV_TRIES		(HL_LOCK_AUTO_LIMIT_HIGH << 5)
+
 //#define READWRITE_SIGNALLING
 
 #define CG_DEV_CONSOLE	"c 5:1"
@@ -239,4 +243,5 @@ expire-after:20M\n\
 #define BSM_QUEUE_LENGTH_MAX		(1024*1024)
 #define GIO_QUEUE_LENGTH_MAX		BSM_QUEUE_LENGTH_MAX
 
+
 #endif
