@@ -1,5 +1,5 @@
 /*
-    clsync - file tree sync utility based on fanotify and inotify
+    clsync - file tree sync utility based on inotify
     
     Copyright (C) 2014  Dmitry Yu Okunev <dyokunev@ut.mephi.ru> 0x8E30679C
     
@@ -22,6 +22,7 @@
 struct clsyncproc {
 	clsyncsock_t 		*sock_p;
 	clsyncsock_procfunct_t	 procfunct;
+	void			*data;
 };
 typedef struct clsyncproc clsyncproc_t;
 
