@@ -97,7 +97,7 @@ static const struct option long_options[] =
 	{"secure-splitting",	no_argument,		NULL,	SECURESPLITTING},
 # endif
 	{"splitting",		required_argument,	NULL,	SPLITTING},
-	{"check-execvp-args",	optional_argument,	NULL,	CHECK_EXECVP_ARGS},
+	{"check-execvp-arguments",optional_argument,	NULL,	CHECK_EXECVP_ARGS},
 	{"add-permitted-hook-files",required_argument,	NULL,	ADDPERMITTEDHOOKFILES},
 # ifdef SECCOMP_SUPPORT
 	{"seccomp-filter",	optional_argument,	NULL,	SECCOMP_FILTER},
@@ -106,8 +106,10 @@ static const struct option long_options[] =
 	{"permit-mprotect",	optional_argument,	NULL,	PERMIT_MPROTECT},
 	{"shm-mprotect",	optional_argument,	NULL,	SHM_MPROTECT},
 #endif
-#ifdef GETMNTENT_SUPPORT
+#ifdef UNSHARE_SUPPORT
+# ifdef GETMNTENT_SUPPORT
 	{"mountpoints",		required_argument,	NULL,	MOUNTPOINTS},
+# endif
 #endif
 #ifdef CAPABILITIES_SUPPORT
 	{"preserve-capabilities",required_argument,	NULL,	CAP_PRESERVE},
@@ -117,7 +119,9 @@ static const struct option long_options[] =
 	{"forbid-devices",	optional_argument,	NULL,	FORBIDDEVICES},
 	{"cgroup-group-name",	required_argument,	NULL,	CG_GROUPNAME},
 #endif
+#ifdef THREADING_SUPPORT
 	{"threading",		required_argument,	NULL,	THREADING},
+#endif
 	{"retries",		required_argument,	NULL,	RETRIES},
 	{"ignore-failures",	optional_argument,	NULL,	IGNOREFAILURES},
 	{"exit-on-sync-skipping",optional_argument,	NULL,	EXITONSYNCSKIP},
@@ -305,12 +309,14 @@ static char *const socketauth[] = {
 	NULL
 };
 
+#ifdef THREADING_SUPPORT
 static char *const threading_modes[] = {
 	[PM_OFF]		= "off",
 	[PM_SAFE]		= "safe",
 	[PM_FULL]		= "full",
 	NULL
 };
+#endif
 
 #ifdef CAPABILITIES_SUPPORT
 static char *const splitting_modes[] = {
@@ -386,7 +392,7 @@ int syntax() {
 }
 
 int ncpus;
-pid_t parent_pid;
+pid_t parent_pid = 0;
 
 pid_t waitpid_timed(pid_t child_pid, int *status_p, long sec, long nsec) {
 	struct timespec ts;
@@ -535,6 +541,9 @@ int version() {
 #ifdef TRE_SUPPORT
 		" -DTRE_SUPPORT"
 #endif
+#ifdef THREADING_SUPPORT
+		" -DTHREADING_SUPPORT"
+#endif
 #ifdef HL_LOCKS
 		" -DHL_LOCKS"
 #endif
@@ -1241,7 +1250,8 @@ static int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsour
 			break;
 		}
 #endif
-#ifdef GETMNTENT_SUPPORT
+#ifdef UNSHARE_SUPPORT
+# ifdef GETMNTENT_SUPPORT
 		case MOUNTPOINTS: {
 			char *ptr;
 			if (paramsource == PS_CONTROL) {
@@ -1287,6 +1297,7 @@ static int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsour
 			}
 			break;
 		}
+# endif
 #endif
 		case PIDFILE:
 			if (paramsource == PS_CONTROL) {
@@ -1298,6 +1309,7 @@ static int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsour
 		case RETRIES:
 			ctx_p->retries		= (unsigned int)xstrtol(arg, &ret);
 			break;
+#ifdef THREADING_SUPPORT
 		case THREADING: {
 			char *value, *arg_orig = arg;
 
@@ -1316,6 +1328,7 @@ static int parse_parameter(ctx_t *ctx_p, uint16_t param_id, char *arg, paramsour
 
 			break;
 		}
+#endif
 		case OUTPUT_METHOD: {
 			char *value, *arg_orig = arg;
 
@@ -1849,6 +1862,13 @@ int ctx_check(ctx_t *ctx_p) {
 	struct utsname utsname;
 #endif
 
+#ifndef _DEBUG_SUPPORT
+	if (ctx_p->flags[DEBUG]) {
+		ret = errno = EINVAL;
+		error("Clsync was compiled without debugging support, please recompile with --enable-debug in order to be able to use debugging");
+	}
+#endif
+
 	if (ctx_p->socketpath != NULL) {
 #ifndef ENABLE_SOCKET
 		ret = EINVAL;
@@ -1879,15 +1899,12 @@ int ctx_check(ctx_t *ctx_p) {
 		error("\"--pivot-root\" cannot be used without \"--chroot\"");
 	}
 
+# ifdef UNSHARE_SUPPORT
+#  ifdef GETMNTENT_SUPPORT
 	if ((ctx_p->flags[PIVOT_ROOT] != PW_OFF) && (ctx_p->mountpoints))
 		warning("\"--mountpoints\" is set while \"--pivot-root\" is set, too");
-#endif
-
-#ifdef VERYPARANOID
-	if ((ctx_p->retries != 1) && ctx_p->flags[THREADING]) {
-		ret = errno = EINVAL;
-		error("\"--retries\" values should be equal to \"1\" for this \"--threading\" value.");
-	}
+#  endif
+# endif
 #endif
 
 	if (ctx_p->flags[STANDBYFILE] && (ctx_p->flags[MODE] == MODE_SIMPLE)) {
@@ -1895,6 +1912,13 @@ int ctx_check(ctx_t *ctx_p) {
 		error("Sorry but option \"--standby-file\" cannot be used in mode \"simple\", yet.");
 	}
 
+#ifdef THREADING_SUPPORT
+# ifdef VERYPARANOID
+	if ((ctx_p->retries != 1) && ctx_p->flags[THREADING]) {
+		ret = errno = EINVAL;
+		error("\"--retries\" values should be equal to \"1\" for this \"--threading\" value.");
+	}
+# endif
 	if (ctx_p->flags[THREADING] && ctx_p->flags[ONLYINITSYNC]) {
 		ret = errno = EINVAL;
 		error("Conflicting options: This value of \"--threading\" cannot be used in conjunction with \"--only-initialsync\".");
@@ -1916,6 +1940,13 @@ int ctx_check(ctx_t *ctx_p) {
 		ret = errno = EINVAL;
 		error("Conflicting options: This value of \"--threading\" cannot be used in conjunction with \"--splitting=thread\".");
 	}
+# ifdef SECCOMP_SUPPORT
+	if (ctx_p->flags[THREADING] && ctx_p->flags[SECCOMP_FILTER]) {
+		ret = errno = EINVAL;
+		error("Conflicting options: This value of \"--threading\" cannot be used in conjunction with \"--seccomp-filter\".");
+	}
+# endif
+#endif
 	if (ctx_p->flags[SKIPINITSYNC] && ctx_p->flags[EXITONNOEVENTS]) {
 		ret = errno = EINVAL;
 		error("Conflicting options: \"--skip-initialsync\" and \"--exit-on-no-events\" cannot be used together.");
@@ -1980,7 +2011,7 @@ int ctx_check(ctx_t *ctx_p) {
 
 	if ((ctx_p->flags[MODE] == MODE_RSYNCDIRECT) && (ctx_p->destdir == NULL)) {
 		ret = errno = EINVAL;
-		error("Mode \"rsyncdirect\" cannot be used without specifying \"--dest-dir\".");
+		error("Mode \"rsyncdirect\" cannot be used without specifying \"--destination-dir\".");
 	}
 
 #ifdef CLUSTER_SUPPORT
@@ -2173,7 +2204,7 @@ int ctx_check(ctx_t *ctx_p) {
 
 	if (ctx_p->flags[CHECK_EXECVP_ARGS] && (ctx_p->flags[MODE] == MODE_DIRECT)) {
 		ret = errno = EINVAL;
-		error("Options --check-execvp-arguments/--secure-splitting cannot be used in conjuction with --mode=direct (see \"man 1 clsync\": --check-execvp-arguments).");
+		error("Options --check-execvp-arguments/--secure-splitting cannot be used in conjunction with --mode=direct (see \"man 1 clsync\": --check-execvp-arguments).");
 	}
 
 #if 0
@@ -2670,7 +2701,8 @@ int main(int _argc, char *_argv[]) {
 	ctx_p->state = STATE_STARTING;
 
 	{
-#ifdef GETMNTENT_SUPPORT
+#ifdef UNSHARE_SUPPORT
+# ifdef GETMNTENT_SUPPORT
 		struct mntent *ent;
 		FILE *ent_f;
 
@@ -2683,10 +2715,9 @@ int main(int _argc, char *_argv[]) {
 				ret = errno;
 			}
 		}
-#endif
+# endif
 
-#ifdef UNSHARE_SUPPORT
-#define unshare_wrapper(a) \
+# define unshare_wrapper(a) \
 		if (unshare(a)) {\
 			error("Got error from unshare("TOSTR(a)")");\
 			ret = errno;\
@@ -2707,7 +2738,7 @@ int main(int _argc, char *_argv[]) {
 		}
 		if (ctx_p->flags[DETACH_NETWORK] == DN_EVERYWHERE)
 			unshare_wrapper(CLONE_NEWNET);
-#undef unshare_wrapper
+# undef unshare_wrapper
 #endif
 
 		if (ctx_p->chroot_dir != NULL) {
@@ -2763,7 +2794,8 @@ int main(int _argc, char *_argv[]) {
 
 		}
 
-#ifdef GETMNTENT_SUPPORT
+#ifdef UNSHARE_SUPPORT
+# ifdef GETMNTENT_SUPPORT
 		if (ctx_p->mountpoints && (ent_f != NULL)) {
 			// Getting mount-points to be umounted
 			while (NULL != (ent = getmntent(ent_f))) {
@@ -2792,6 +2824,7 @@ int main(int _argc, char *_argv[]) {
 
 			endmntent(ent_f);
 		}
+# endif
 #endif
 		if (ctx_p->chroot_dir != NULL) {
 #ifdef PIVOTROOT_OPT_SUPPORT
