@@ -64,7 +64,15 @@
 # include <linux/filter.h>		// struct sock_filter
 # include <linux/seccomp.h>		// SECCOMP_RET_*
 
-#define syscall_nr (offsetof(struct seccomp_data, nr))
+# ifndef __NR_shmdt
+#  ifdef __i386__
+#   warning [security] Caution! __NR_shmdt is not defined. Setting it to -222.
+#   define __NR_shmdt -222
+#  endif
+# endif
+
+
+# define syscall_nr (offsetof(struct seccomp_data, nr))
 
 /* Read: http://www.rawether.net/support/bpfhelp.htm */
 # define SECCOMP_COPY_SYSCALL_TO_ACCUM				\
@@ -84,19 +92,14 @@
 	SECCOMP_ALLOW_ACCUM_SYSCALL(futex),				\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(inotify_init1),			\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(alarm),				\
-	SECCOMP_ALLOW_ACCUM_SYSCALL(stat),		/* unused */	\
-	SECCOMP_ALLOW_ACCUM_SYSCALL(fstat),		/* unused */	\
-	SECCOMP_ALLOW_ACCUM_SYSCALL(lstat),				\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(open),				\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(write),				\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(close),				\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(wait4),				\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(unlink),				\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(tgkill),				\
-	SECCOMP_ALLOW_ACCUM_SYSCALL(clock_gettime),			\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(rt_sigreturn),			\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(brk),				\
-	SECCOMP_ALLOW_ACCUM_SYSCALL(mmap),				\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(munmap),				\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(wait4),				\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(rmdir),				\
@@ -111,18 +114,39 @@
 	SECCOMP_ALLOW_ACCUM_SYSCALL(set_robust_list),	/* for --threading? */		\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(madvise),				\
 	SECCOMP_ALLOW_ACCUM_SYSCALL(exit),				\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(clock_gettime),			\
 
+# ifdef __i386__
+#  define FILTER_TABLE_ARCHDEPENDED			/* unused */	\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(fstat64),				\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(lstat64),				\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(stat64),				\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(time),				\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(mmap2),				\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(gettimeofday),			\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(_newselect),			\
+
+# else
+#  define FILTER_TABLE_ARCHDEPENDED					\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(fstat),		/* unused */	\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(lstat),				\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(stat),		/* unused */	\
+	SECCOMP_ALLOW_ACCUM_SYSCALL(mmap),				\
+
+# endif
 
 
 /* Syscalls allowed to non-privileged thread */
 static struct sock_filter filter_table[] = {
 	SECCOMP_COPY_SYSCALL_TO_ACCUM,
 	FILTER_TABLE_NONPRIV
+	FILTER_TABLE_ARCHDEPENDED
 	SECCOMP_DENY,
 };
 static struct sock_filter filter_w_mprotect_table[] = {
 	SECCOMP_COPY_SYSCALL_TO_ACCUM,
 	FILTER_TABLE_NONPRIV
+	FILTER_TABLE_ARCHDEPENDED
 	SECCOMP_ALLOW_ACCUM_SYSCALL(mprotect),
 	SECCOMP_DENY,
 };
@@ -177,6 +201,8 @@ enum privileged_action {
 
 	PA_DIE,
 
+	PA_LSTAT64,
+
 	PA_FTS_OPEN,
 	PA_FTS_READ,
 	PA_FTS_CLOSE,
@@ -195,6 +221,11 @@ enum privileged_action {
 	PA_WAITPID,
 };
 
+struct pa_lstat64_arg {
+	char		 path_buf[PATH_MAX];
+	const char	*path;
+};
+
 struct pa_fts_open_arg {
 	char		_path_argv[MAXARGUMENTS+1][PATH_MAX];
 	char		*path_argv[MAXARGUMENTS+1];
@@ -236,6 +267,7 @@ struct pa_waitpid_arg {
 };
 
 struct pa_arg {
+	struct pa_lstat64_arg		 lstat64;
 	struct pa_fts_open_arg		 fts_open;
 	struct pa_inotify_add_watch_arg	 inotify_add_watch;
 	struct pa_inotify_rm_watch_arg	 inotify_rm_watch;
@@ -292,7 +324,7 @@ struct pa_fts_read_ret {
 	char		fts_name[PATH_MAX];
 };
 struct pa_ret {
-	struct stat		stat;
+	stat64_t		stat64;
 	struct pa_fts_read_ret	fts_read;
 };
 struct cmd {
@@ -306,6 +338,7 @@ struct cmd_ret {
 	volatile struct pa_ret		 ret_buf;
 	volatile void			*ret;
 	volatile int			 _errno;
+	volatile int			 processing_longcmd;
 };
 volatile struct cmd		*cmd_p;
 volatile struct cmd_ret		*cmd_ret_p;
@@ -317,6 +350,7 @@ volatile struct hl_lock		*hl_lock_p;
 static inline void hl_lock_init(volatile struct hl_lock *hl_lock_p) {
 	debug(10, "");
 	hl_lock_p->enabled = 1;
+	hl_lock_p->locallock_hl_setstate_ifstate = 1;
 #  ifdef HL_LOCK_TRIES_AUTO
 	int i;
 	i = 0;
@@ -344,6 +378,13 @@ struct pa_options {
 	int   shm_mprotect;
 };
 
+int (*_privileged_lstat64)		(
+		const char *path, stat64_t *buf
+# ifdef HL_LOCK_TRIES_AUTO
+		, int callid
+# endif
+	);
+
 FTS *(*_privileged_fts_open)		(
 		char * const *path_argv,
 		int options,
@@ -655,6 +696,9 @@ int pa_setup(struct pa_options *opts, ctx_t *ctx_p, uid_t *exec_uid_p, gid_t *ex
 }
 
 int pa_unsetup(struct pa_options *opts) {
+#ifdef TODO_FIX
+	// segfaults: gdb --args clsync -K lxc-brother-atomic-sync -l jabber --pre-exit-hook wlxc-stop --chroot= --pivot-root=off -d9 -b0 -Ystderr
+
 	free(opts->exithookfile);
 	free(opts->preexithookfile);
 	free(opts->label);
@@ -679,10 +723,44 @@ int pa_unsetup(struct pa_options *opts) {
 			i++;
 		}
 	}
+#endif
 
 	return 0;
 }
 
+static int helper_isalive_cache;
+static inline int helper_isalive_proc() {
+	int rc;
+	debug(12, "helper_pid == %u", helper_pid);
+
+	if ((rc=waitpid(helper_pid, NULL, WNOHANG))>=0)
+		return helper_isalive_cache=1;
+
+	debug(1, "waitpid(%u, NULL, WNOHANG) => %i", helper_pid, rc);
+
+	return helper_isalive_cache=0;
+}
+static inline int helper_isalive_thread() {
+	int rc;
+	debug(12, "");
+
+	if ((rc=pthread_kill(privileged_thread, 0)))
+		return helper_isalive_cache=0;
+
+	debug(1, "pthread_kill(privileged_thread, 0) => %i", helper_pid, rc);
+
+	return helper_isalive_cache=1;
+}
+static inline int helper_isalive() {
+	return helper_pid ? helper_isalive_proc() : helper_isalive_thread();
+}
+
+int privileged_check() {
+	if (helper_pid)
+		critical_on(!helper_isalive_proc());
+	return 0;
+}
+
 # ifdef HL_LOCKS
 
 static inline int hl_isanswered(int lockid) {
@@ -698,29 +776,33 @@ static inline void hl_setstate(int lockid, hllock_state_t stateid) {
 }
 
 int hl_setstate_ifstate(int lockid, hllock_state_t stateid_new, hllock_state_t stateid_old_mask) {
+	static long long counter = 0;
 	volatile int *local_lock_p = &hl_lock_p->locallock_hl_setstate_ifstate;
 	debug(90, "%i, 0x%o, 0x%o", lockid, stateid_new, stateid_old_mask);
 
-	if (*local_lock_p)
-		return 0;
+	if (helper_pid || parent_pid) {
+		if (((++counter) & 0xffffff) == 0) {
+			if (helper_pid) {
+				critical_on(!helper_isalive_proc());
+			} else {
+				critical_on(!parent_isalive());
+			}
+		}
+	}
 
-	debug(92, "%i", *local_lock_p);
-	g_atomic_int_inc(local_lock_p);
-	debug(92, "%i", *local_lock_p);
-	if (*local_lock_p != 1) {
-		g_atomic_int_dec_and_test(local_lock_p);
+	if (!g_atomic_int_dec_and_test(local_lock_p)) {
+		g_atomic_int_inc(local_lock_p);
 		return 0;
 	}
 
 	if (!(hl_lock_p->state[lockid]&stateid_old_mask)) {
-		g_atomic_int_dec_and_test(local_lock_p);
+		g_atomic_int_inc(local_lock_p);
 		return 0;
 	}
 
 	debug(50, "success");
 	g_atomic_int_set(&hl_lock_p->state[lockid], stateid_new);
-	g_atomic_int_dec_and_test(local_lock_p);
-#undef local_lock
+	g_atomic_int_inc(local_lock_p);
 	return 1;
 }
 
@@ -791,39 +873,6 @@ void hl_shutdown(int lockid) {
 
 # endif
 
-static int helper_isalive_cache;
-static inline int helper_isalive_proc() {
-	int rc;
-	debug(12, "helper_pid == %u", helper_pid);
-
-	if ((rc=waitpid(helper_pid, NULL, WNOHANG))>=0)
-		return helper_isalive_cache=1;
-
-	debug(1, "waitpid(%u, NULL, WNOHANG) => %i", helper_pid, rc);
-
-	return helper_isalive_cache=0;
-}
-static inline int helper_isalive_thread() {
-	int rc;
-	debug(12, "");
-
-	if ((rc=pthread_kill(privileged_thread, 0)))
-		return helper_isalive_cache=0;
-
-	debug(1, "pthread_kill(privileged_thread, 0) => %i", helper_pid, rc);
-
-	return helper_isalive_cache=1;
-}
-static inline int helper_isalive() {
-	return helper_pid ? helper_isalive_proc() : helper_isalive_thread();
-}
-
-int privileged_check() {
-	if (helper_pid)
-		critical_on(!helper_isalive_proc());
-	return 0;
-}
-
 int privileged_handler(ctx_t *ctx_p)
 {
 # ifdef READWRITE_SIGNALLING
@@ -965,6 +1014,18 @@ int privileged_handler(ctx_t *ctx_p)
 				debug(20, "PA_DIE");
 				helper_isrunning = 0;
 				break;
+			case PA_LSTAT64: {
+				volatile struct pa_lstat64_arg *arg_p = &cmd_p->arg.lstat64;
+				stat64_t *ret_buf = (void *)&cmd_ret_p->ret_buf.stat64;
+				volatile const char *path =	opts->isprocsplitting 	?
+								arg_p->path_buf 	:
+								arg_p->path;
+
+				debug(20, "PA_LSTAT64 (%s)", path);
+				cmd_ret_p->ret = (void *)(long)lstat64((const char *)path, ret_buf);
+				debug(21, "/PA_LSTAT64 => %i", (int)(long)cmd_ret_p->ret);
+				break;
+			}
 			case PA_FTS_OPEN: {
 				volatile struct pa_fts_open_arg *arg_p = &cmd_p->arg.fts_open;
 				char *const *path_argv_p =(void *)(
@@ -1054,19 +1115,28 @@ int privileged_handler(ctx_t *ctx_p)
 					case -1: 
 						error("Cannot fork().");
 						break;
-					case  0:
+					case  0: {
+						int rc;
+						(void)rc;	// anti-warning on ./configure --enable-debug=no
 #ifdef ANTIPARANOID
 						if (ctx_p->privileged_gid != exec_gid)
 #endif
-							debug(4, "setgid(%u) == %i", exec_gid, setgid(exec_gid));
+						{
+							rc = setgid(exec_gid);
+							debug(4, "setgid(%u) == %i", exec_gid, rc);
+						}
 
 #ifdef ANTIPARANOID
 						if (ctx_p->privileged_uid != exec_uid)
 #endif
-							debug(4, "setuid(%u) == %i", exec_uid, setuid(exec_uid));
+						{
+							rc = setuid(exec_uid);
+							debug(4, "setuid(%u) == %i", exec_uid, rc);
+						}
 
 						debug(3, "execvp(\"%s\", argv)", file);
 						exit(execvp(file, argv));
+					}
 				}
 				cmd_ret_p->ret = (void *)(long)pid;
 				debug(21, "/PA_FORK_EXECVP");
@@ -1125,7 +1195,9 @@ int privileged_handler(ctx_t *ctx_p)
 			case PA_WAITPID: {
 				struct pa_waitpid_arg *arg_p = (void *)&cmd_p->arg.waitpid;
 				debug(20, "PA_WAITPID(%u, 0x%o)", arg_p->pid, arg_p->options);
-				cmd_ret_p->ret = (void *)(long)waitpid(arg_p->pid, &arg_p->status, arg_p->options);
+				cmd_ret_p->processing_longcmd = 1;
+				cmd_ret_p->ret                = (void *)(long)waitpid(arg_p->pid, &arg_p->status, arg_p->options);
+				cmd_ret_p->processing_longcmd = 0;
 				break;
 			}
 			default:
@@ -1190,12 +1262,17 @@ static inline int privileged_action(
 	debug(10, "Waiting the privileged thread/process to get prepared for signal");
 #  ifdef HL_LOCKS
 	if (hl_lock_p->enabled) {
-		while (!hl_isanswered(HLLOCK_HANDLER))
+		long long counter = 0;
+
+		while (!hl_isanswered(HLLOCK_HANDLER)) {
 			if (!helper_isalive_cache) {
 				debug(1, "The privileged thread/process is dead (#0). Ignoring the command.");
 				rc = ENOENT;
 				goto privileged_action_end;
 			}
+			if (cmd_ret_p->processing_longcmd && ++counter > HL_LOCK_NONPRIV_TRIES)
+				sleep(SLEEP_SECONDS);
+		}
 	} else {
 #  endif
 		critical_on(!helper_isalive_cache);
@@ -1260,12 +1337,17 @@ static inline int privileged_action(
 
 # ifdef HL_LOCKS
 	if (hl_lock_p->enabled) {
-		while (!hl_isanswered(HLLOCK_HANDLER))
+		long long counter = 0;
+
+		while (!hl_isanswered(HLLOCK_HANDLER)) {
 			if (!helper_isalive_cache) {
 				debug(1, "The privileged thread/process is dead (#2). Ignoring the command.");
 				rc = ENOENT;
 				goto privileged_action_end;
 			}
+			if (cmd_ret_p->processing_longcmd && ++counter > HL_LOCK_NONPRIV_TRIES)
+				sleep(SLEEP_SECONDS);
+		}
 
 #  ifdef HL_LOCK_TRIES_AUTO
 		if (isadjusting) {
@@ -1321,6 +1403,54 @@ privileged_action_end:
 	return rc;
 }
 
+int __privileged_lstat64_procsplit(
+		const char *path, stat64_t *buf
+# ifdef HL_LOCK_TRIES_AUTO
+		, int callid
+# endif
+	)
+{
+	void *ret = (void *)(long)-1;
+
+	strcpy((char *)cmd_p->arg.lstat64.path_buf, path);
+
+	privileged_action(
+# ifdef HL_LOCK_TRIES_AUTO
+			callid,
+# endif
+			PA_LSTAT64,
+			&ret
+		);
+
+	memcpy(buf, (void *)&cmd_ret_p->ret_buf.stat64, sizeof(stat64_t));
+
+	return (long)ret;
+}
+
+int __privileged_lstat64_threadsplit(
+		const char *path, stat64_t *buf
+# ifdef HL_LOCK_TRIES_AUTO
+		, int callid
+# endif
+	)
+{
+	void *ret = (void *)(long)-1;
+
+	cmd_p->arg.lstat64.path = path;
+
+	privileged_action(
+# ifdef HL_LOCK_TRIES_AUTO
+			callid,
+# endif
+			PA_LSTAT64,
+			&ret
+		);
+
+	memcpy(buf, (void *)&cmd_ret_p->ret_buf.stat64, sizeof(stat64_t));
+
+	return (long)ret;
+}
+
 FTS *__privileged_fts_open_procsplit(
 		char *const *path_argv,
 		int options,
@@ -1640,12 +1770,20 @@ int __privileged_fork_execvp(const char *file, char *const argv[])
 		case -1: 
 			error("Cannot fork().");
 			return -1;
-		case  0:
-			debug(4, "setgid(%u) == %i", __privileged_fork_execvp_gid, setgid(__privileged_fork_execvp_gid));
-			debug(4, "setuid(%u) == %i", __privileged_fork_execvp_uid, setuid(__privileged_fork_execvp_uid));
+		case  0: {
+			int rc;
+			(void)rc;	// anti-warning on ./configure --enable-debug=no
+
+			rc = setgid(__privileged_fork_execvp_gid);
+			debug(4, "setgid(%u) == %i", __privileged_fork_execvp_gid, rc);
+
+			rc = setuid(__privileged_fork_execvp_uid);
+			debug(4, "setuid(%u) == %i", __privileged_fork_execvp_uid, rc);
+
 			errno = 0;
 			execvp(file, argv);
 			exit(errno);
+		}
 	}
 
 	return pid;
@@ -1739,6 +1877,7 @@ int privileged_init(ctx_t *ctx_p)
 		_privileged_kill_child		= __privileged_kill_child_itself;
 
 #ifdef CAPABILITIES_SUPPORT
+		_privileged_lstat64		= (typeof(_privileged_lstat64))			lstat64;
 		_privileged_fts_open		= (typeof(_privileged_fts_open))		fts_open;
 		_privileged_fts_read		= (typeof(_privileged_fts_read))		fts_read;
 		_privileged_fts_close		= (typeof(_privileged_fts_close))		fts_close;
@@ -1796,6 +1935,7 @@ int privileged_init(ctx_t *ctx_p)
 
 	switch (ctx_p->flags[SPLITTING]) {
 		case SM_THREAD: {
+			_privileged_lstat64		= __privileged_lstat64_threadsplit;
 			_privileged_fts_open		= __privileged_fts_open_threadsplit;
 			_privileged_inotify_add_watch	= __privileged_inotify_add_watch_threadsplit;
 			_privileged_fork_execvp		= __privileged_fork_setuid_execvp_threadsplit;
@@ -1815,6 +1955,7 @@ int privileged_init(ctx_t *ctx_p)
 			break;
 		}
 		case SM_PROCESS: {
+			_privileged_lstat64		= __privileged_lstat64_procsplit;
 			_privileged_fts_open		= __privileged_fts_open_procsplit;
 			_privileged_inotify_add_watch	= __privileged_inotify_add_watch_procsplit;
 			_privileged_fork_execvp		= __privileged_fork_setuid_execvp_procsplit;
